# import pytest
# from http import HTTPStatus
# from pbench.server.api.resources.query_apis.search_api import IndexSearchAPI
# from pbench.test.unit.server.query_apis.commons import Commons
#
#
# class TestIndexSearch(Commons):
#     """
#     Unit testing for resources/SearchAPI class.
#
#     In a web service context, we access class functions mostly via the
#     Flask test client rather than trying to directly invoke the class
#     constructor and `post` service.
#     """
#
#     @pytest.fixture(autouse=True)
#     def _setup(self):
#         super(TestIndexSearch, self)._setup(
#             pbench_endpoint="/index/search",
#             elastic_endpoint="/_search?ignore_unavailable=true",
#             payload={
#                 "user": "drb",
#                 "start": "2020-08",
#                 "end": "2020-10",
#                 "search_term": "random_string",
#             },
#             bad_date_payload={
#                 "user": "drb",
#                 "start": "2020-12",
#                 "end": "2020-19",
#                 "search_term": "random_string",
#             },
#             error_payload={"start": "2020-08", "end": "2020-08"},
#             empty_response_payload={
#                 "took": 4,
#                 "timed_out": "false",
#                 "_shards": {"total": 5, "successful": 5, "skipped": 2, "failed": 0},
#                 "hits": {
#                     "total": {"value": 0, "relation": "eq"},
#                     "max_score": "null",
#                     "hits": [],
#                 },
#             },
#         )
#
#     @pytest.mark.parametrize(
#         "keys",
#         (
#             {"user": "x"},
#             {"start": "2020"},
#             {"end": "2020"},
#             {"user": "x", "start": "2020"},
#             {"user": "x", "end": "2020"},
#             {"some_additional_key": "test"},
#         ),
#     )
#     def test_missing_keys(self, client, server_config, keys, user_ok, pbench_token):
#         self.required_keys = [
#             key
#             for key, parameter in IndexSearchAPI(
#                 client.config, client.logger
#             ).schema.parameters.items()
#             if parameter.required
#         ]
#         super().missing_keys(client, server_config, keys, user_ok, pbench_token)
#
#     @pytest.mark.parametrize(
#         "user", ("drb", "badwolf", "no_user"),
#     )
#     def test_query(
#         self,
#         client,
#         server_config,
#         query_api,
#         user_ok,
#         find_template,
#         build_auth_header,
#         user,
#     ):
#         """
#         Check the construction of Elasticsearch query URI and filtering of the response body.
#         The test will run once with each parameter supplied from the local parameterization,
#         and, for each of those, three times with different values of the build_auth_header fixture.
#         """
#         payload = {
#             "user": user,
#             "access": "private",
#             "start": "2021-06",
#             "end": "2021-08",
#             "search_term": "random_string",
#             "fields": [
#                 "host_tools_info.hostname",
#                 "@timestamp",
#                 "run.controller",
#                 "run.name",
#             ],
#         }
#         if user == "no_user":
#             payload.pop("user", None)
#         if user == "no_user" or user is None:
#             payload["access"] = "public"
#
#         response_payload = {
#             "took": 11,
#             "timed_out": "false",
#             "_shards": {"total": 5, "successful": 5, "skipped": 2, "failed": 0},
#             "hits": {
#                 "total": {"value": 3, "relation": "eq"},
#                 "max_score": "null",
#                 "hits": [
#                     {
#                         "_index": "npalaska-pbench.v6.run-data.2021-07",
#                         "_type": "_doc",
#                         "_id": "f3a37c9891a78886639e3bc00e3c5c4e",
#                         "_score": "null",
#                         "_source": {
#                             "@timestamp": "2021-07-14T15:30:23.652778",
#                             "run": {
#                                 "controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                                 "name": "uperf_npalaska-dhcp31-171_2021.07.14T15.30.22",
#                             },
#                         },
#                         "sort": [1626276623652],
#                     },
#                     {
#                         "_index": "npalaska-pbench.v6.run-data.2021-07",
#                         "_type": "_doc",
#                         "_id": "1c25e9f5b5dfc1ffb732931bf3899878",
#                         "_score": "null",
#                         "_source": {
#                             "@timestamp": "2021-07-12T22:44:19.562354",
#                             "run": {
#                                 "controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                                 "name": "pbench-user-benchmark_npalaska-dhcp31-171_2021.07.12T22.44.19",
#                             },
#                             "host_tools_info": [{"hostname": "dhcp31-171"}],
#                         },
#                         "sort": [1626129859562],
#                     },
#                     {
#                         "_index": "npalaska-pbench.v6.run-data.2021-07",
#                         "_type": "_doc",
#                         "_id": "6e9a82d1167ed1dc4053c3654fe8af13",
#                         "_score": "null",
#                         "_source": {
#                             "@timestamp": "2021-07-12T14:08:46.563303",
#                             "run": {
#                                 "controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                                 "name": "pbench-user-benchmark_npalaska-dhcp31-171_2021.07.12T14.08.46",
#                             },
#                             "host_tools_info": [{"hostname": "dhcp31-171"}],
#                         },
#                         "sort": [1626098926563],
#                     },
#                 ],
#             },
#         }
#         index = self.build_index(server_config, ("2021-06", "2021-07", "2021-08"))
#         # If we're not asking about a particular user, or if the user
#         # field is to be omitted altogether, or if we have a valid
#         # token, then the request should succeed.
#         if (
#             not user
#             or user == "no_user"
#             or build_auth_header["header_param"] == "valid"
#         ) and user != "badwolf":
#             expected_status = HTTPStatus.OK
#         else:
#             expected_status = HTTPStatus.FORBIDDEN
#
#         response = query_api(
#             "/index/search",
#             "/_search?ignore_unavailable=true",
#             payload,
#             index,
#             expected_status,
#             headers=build_auth_header["header"],
#             status=expected_status,
#             json=response_payload,
#         )
#         assert response.status_code == expected_status
#         if response.status_code == HTTPStatus.OK:
#             res_json = response.json
#             assert isinstance(res_json, list)
#             assert len(res_json) == 3
#             expected_result = {
#                 "host_tools_info.hostname": "dhcp31-171",
#                 "@timestamp": "2021-07-12T22:44:19.562354",
#                 "id": "1c25e9f5b5dfc1ffb732931bf3899878",
#                 "run.controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                 "run.name": "pbench-user-benchmark_npalaska-dhcp31-171_2021.07.12T22.44.19",
#             }
#             assert res_json[1] == expected_result
#
#     @pytest.mark.parametrize(
#         "test_json, expected_result",
#         [
#             (
#                 {
#                     "@timestamp": "2021-07-12T14:08:46.563303",
#                     "run": {
#                         "controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                         "name": "pbench-user-benchmark_npalaska-dhcp31-171_2021.07.12T14.08.46",
#                     },
#                     "host_tools_info": [
#                         {"hostname": [{"test": "dhcp31-171"}, {"test2": "dhcp31-195"}]}
#                     ],
#                 },
#                 {
#                     "@timestamp": "2021-07-12T14:08:46.563303",
#                     "run.controller": "dhcp31-171.perf.lab.eng.bos.redhat.com",
#                     "run.name": "pbench-user-benchmark_npalaska-dhcp31-171_2021.07.12T14.08.46",
#                     "host_tools_info.hostname.test": "dhcp31-171",
#                     "host_tools_info.hostname.test2": "dhcp31-195",
#                 },
#             ),
#             (
#                 {
#                     "a": {"b": 1, "c": None, "d": [False, True, {"c": "test"}]},
#                     "e": "f",
#                     "g": 2.3,
#                 },
#                 {
#                     "a.b": 1,
#                     "a.c": None,
#                     "a.d.0": False,
#                     "a.d.1": True,
#                     "a.d.c": "test",
#                     "e": "f",
#                     "g": 2.3,
#                 },
#             ),
#             ({"": 1}, {"": 1}),
#         ],
#     )
#     def test_flatten_json(self, client, test_json, expected_result):
#         searchapi = IndexSearchAPI(client.config, client.logger)
#         assert searchapi.flatten_json(test_json) == expected_result
