import click

from pbench import BadConfig
from pbench.cli.server import pass_cli_context
from pbench.cli.server.options import common_options
from pbench.common.logger import get_pbench_logger
from pbench.server import PbenchServerConfig
from pbench.server.database.database import Database
from pbench.server.database.models.users import User

_NAME_ = "pbench-user-delete"


class UserDelete:
    def __init__(self, context):
        self.context = context

    def execute(self):
        config = PbenchServerConfig(self.context.config)

        logger = get_pbench_logger(_NAME_, config)

        # We're going to need the Postgres DB to track dataset state, so setup
        # DB access.
        Database.init_db(config, logger)

        # Delete the the user with specified username
        User.delete(username=self.context.username)

        click.echo(f"User {self.context.username} deleted")


@click.command()
@common_options
@click.option(
    "--username",
    required=True,
    prompt=True,
    help="pbench server account username to be deleted (will prompt if unspecified)",
)
@pass_cli_context
def main(context: object, username: str) -> None:
    context.username = username

    try:
        rv = UserDelete(context).execute()
    except Exception as exc:
        click.echo(exc, err=True)
        rv = 2 if isinstance(exc) == BadConfig else 1

    click.get_current_context().exit(rv)
