import click

from pbench import BadConfig
from pbench.cli.server import pass_cli_context
from pbench.cli.server.options import common_options
from pbench.common.logger import get_pbench_logger
from pbench.server import PbenchServerConfig
from pbench.server.database.database import Database
from pbench.server.database.models.users import User

_NAME_ = "pbench-user-update"


class UserUpdate:
    def __init__(self, context):
        self.context = context

    def execute(self):
        config = PbenchServerConfig(self.context.config)

        logger = get_pbench_logger(_NAME_, config)

        # We're going to need the Postgres DB to track dataset state, so setup
        # DB access.
        Database.init_db(config, logger)

        dict_to_update = {}
        if self.context.username:
            dict_to_update["username"] = self.context.username

        if self.context.first_name:
            dict_to_update["first_name"] = self.context.first_name

        if self.context.last_name:
            dict_to_update["last_name"] = self.context.last_name

        if self.context.email:
            dict_to_update["email"] = self.context.email

        if self.context.role:
            dict_to_update["role"] = self.context.role

        # If the authorization passed update the user from the given username
        User.update(dict_to_update)

        click.echo(f"User {self.context.user_to_update} updated")


@click.command()
@common_options
@click.option(
    "--user-to-update",
    prompt=True,
    required=True,
    help="Specify username of a pbench server account user to update (will prompt if unspecified)",
)
@click.option(
    "--username", required=False, help="Specify the new username",
)
@click.option(
    "--email", required=False, help="Specify the new email",
)
@click.option(
    "--first-name", required=False, help="Specify the new first name",
)
@click.option(
    "--last-name", required=False, help="Specify the new last name",
)
@click.option(
    "--role", required=False, help="Specify the new role",
)
@pass_cli_context
def main(
    context: object,
    user: str,
    user_to_update: str,
    username: str,
    first_name: str,
    last_name: str,
    email: str,
    role: str,
) -> None:
    context.user_to_update = user_to_update
    context.username = username
    context.first_name = first_name
    context.last_name = last_name
    context.email = email
    context.role = role

    try:
        rv = UserUpdate(context).execute()
    except Exception as exc:
        click.echo(exc, err=True)
        rv = 2 if isinstance(exc) == BadConfig else 1

    click.get_current_context().exit(rv)
