import click

from pbench import BadConfig
from pbench.cli.server import pass_cli_context
from pbench.cli.server.options import common_options
from pbench.common.logger import get_pbench_logger
from pbench.server import PbenchServerConfig
from pbench.server.database.database import Database
from pbench.server.database.models.users import User

_NAME_ = "pbench-user-list"


class UsersList:
    def __init__(self, context):
        self.context = context

    def execute(self):
        config = PbenchServerConfig(self.context.config)

        logger = get_pbench_logger(_NAME_, config)

        # We're going to need the Postgres DB to track dataset state, so setup
        # DB access.
        Database.init_db(config, logger)

        click.echo(f"First_Name Last_Name Registered_On Email")
        if not self.context.reversed:
            users = Database.db_session.query(User).limit(self.context.N).all()
        else:
            users = (
                Database.db_session.query(User)
                .order_by(User.id.desc())
                .limit(self.context.N)
                .all()
            )

        for user in users:
            click.echo(
                f"{user.first_name} {user.last_name} {user.registered_on} {user.email}"
            )


@click.command()
@common_options
@click.option(
    "--N",
    required=False,
    default=100,
    help="Number of user table rows to print, defaults to 100",
)
@click.option(
    "--reversed",
    type=bool,
    required=False,
    default=False,
    help="Last n rows of user table to print",
)
@pass_cli_context
def main(context: object, N: int, reversed: bool) -> None:
    context.N = N
    context.reversed = reversed
    try:
        rv = UsersList(context).execute()
    except Exception as exc:
        click.echo(exc, err=True)
        rv = 2 if isinstance(exc) == BadConfig else 1

    click.get_current_context().exit(rv)
